@@ -41,6 +41,7 @@ import struct
 import struct
 import StringIO
 import random
+from keystore import xpubkey_to_address
 
 NO_SIGNATURE = 'ff'
 
@@ -291,38 +292,6 @@ def parse_sig(x_sig):
             s.append(None)
     return s
 
-def is_extended_pubkey(x_pubkey):
-    return x_pubkey[0:2] in ['fe', 'ff']
-
-def x_to_xpub(x_pubkey):
-    if x_pubkey[0:2] == 'ff':
-        from account import BIP32_Account
-        xpub, s = BIP32_Account.parse_xpubkey(x_pubkey)
-        return xpub
-
-
-
-def parse_xpub(x_pubkey):
-    if x_pubkey[0:2] in ['02','03','04']:
-        pubkey = x_pubkey
-    elif x_pubkey[0:2] == 'ff':
-        from account import BIP32_Account
-        xpub, s = BIP32_Account.parse_xpubkey(x_pubkey)
-        pubkey = BIP32_Account.derive_pubkey_from_xpub(xpub, s[0], s[1])
-    elif x_pubkey[0:2] == 'fe':
-        from account import OldAccount
-        mpk, s = OldAccount.parse_xpubkey(x_pubkey)
-        pubkey = OldAccount.get_pubkey_from_mpk(mpk.decode('hex'), s[0], s[1])
-    elif x_pubkey[0:2] == 'fd':
-        addrtype = ord(x_pubkey[2:4].decode('hex'))
-        hash160 = x_pubkey[4:].decode('hex')
-        pubkey = None
-        address = hash_160_to_bc_address(hash160, addrtype)
-    else:
-        raise BaseException("Cannnot parse pubkey")
-    if pubkey:
-        address = public_key_to_bc_address(pubkey.decode('hex'))
-    return pubkey, address
 
 
 def parse_scriptSig(d, bytes):
@@ -353,7 +322,7 @@ def parse_scriptSig(d, bytes):
         x_pubkey = decoded[1][1].encode('hex')
         try:
             signatures = parse_sig([sig])
-            pubkey, address = parse_xpub(x_pubkey)
+            pubkey, address = xpubkey_to_address(x_pubkey)
         except:
             import traceback
             traceback.print_exc(file=sys.stdout)
@@ -382,7 +351,7 @@ def parse_scriptSig(d, bytes):
         print_error("cannot find address in input script", bytes.encode('hex'))
         return
     x_pubkeys = map(lambda x: x[1].encode('hex'), dec2[1:-2])
-    pubkeys = [parse_xpub(x)[0] for x in x_pubkeys]     # xpub, addr = parse_xpub()
+    pubkeys = [xpubkey_to_address(x)[0] for x in x_pubkeys]
     redeemScript = Transaction.multisig_script(pubkeys, m)
     # write result in d
     d['num_sig'] = m
@@ -489,6 +458,7 @@ class Transaction:
             raise BaseException("cannot initialize transaction", raw)
         self._inputs = None
         self._outputs = None
+        self.locktime = 0
 
     def update(self, raw):
         self.raw = raw
@@ -561,7 +531,7 @@ class Transaction:
         for privkey in privkeys:
             pubkey = public_key_from_private_key(privkey)
             address = address_from_private_key(privkey)
-            u = network.synchronous_get(('blockchain.address.listunspent',[address]))
+            u = network.synchronous_get(('blockchain.address.listunspent', [address]))
             pay_script = klass.pay_script(TYPE_ADDRESS, address)
             for item in u:
                 item['scriptPubKey'] = pay_script
@@ -669,11 +639,15 @@ class Transaction:
         s += int_to_hex(txin['prevout_n'], 4)
         # Script length, script, sequence
         script = self.input_script(txin, i, for_sig)
-        s += var_int(len(script) / 2)
+        s += var_int(len(script)/2)
         s += script
-        s += "ffffffff"
+        s += int_to_hex(txin.get('sequence', 0xffffffff), 4)
         return s
 
+    def set_sequence(self, n):
+        for txin in self.inputs():
+            txin['sequence'] = n
+
     def BIP_LI01_sort(self):
         # See https://github.com/kristovatlas/rfc/blob/master/bips/bip-li01.mediawiki
         self._inputs.sort(key = lambda i: (i['prevout_hash'], i['prevout_n']))
@@ -682,27 +656,27 @@ class Transaction:
     def serialize(self, for_sig=None):
         inputs = self.inputs()
         outputs = self.outputs()
-        s  = int_to_hex(1,4)                                         # version
-        s += var_int( len(inputs) )                                  # number of inputs
+        s = int_to_hex(1, 4)                                         # version
+        s += var_int(len(inputs))                                    # number of inputs
         for i, txin in enumerate(inputs):
             s += self.serialize_input(txin, i, for_sig)
-        s += var_int( len(outputs) )                                 # number of outputs
+        s += var_int(len(outputs))                                   # number of outputs
         for output in outputs:
             output_type, addr, amount = output
-            s += int_to_hex( amount, 8)                              # amount
+            s += int_to_hex(amount, 8)                               # amount
             script = self.pay_script(output_type, addr)
-            s += var_int( len(script)/2 )                           #  script length
-            s += script                                             #  script
-        s += int_to_hex(0,4)                                        #  lock time
+            s += var_int(len(script)/2)                              #  script length
+            s += script                                              #  script
+        s += int_to_hex(self.locktime, 4)                            #  locktime
         if for_sig is not None and for_sig != -1:
-            s += int_to_hex(1, 4)                                   #  hash type
+            s += int_to_hex(1, 4)                                    #  hash type
         return s
 
     def tx_for_sig(self,i):
         return self.serialize(for_sig = i)
 
     def hash(self):
-        return Hash(self.raw.decode('hex') )[::-1].encode('hex')
+        return Hash(self.raw.decode('hex'))[::-1].encode('hex')
 
     def add_inputs(self, inputs):
         self._inputs.extend(inputs)
@@ -722,7 +696,7 @@ class Transaction:
         return self.input_value() - self.output_value()
 
     def is_final(self):
-        return not any([x.get('sequence') < 0xffffffff - 1 for x in self.inputs()])
+        return not any([x.get('sequence', 0xffffffff) < 0xffffffff - 1 for x in self.inputs()])
 
     @profiler
     def estimated_size(self):
@@ -756,23 +730,6 @@ class Transaction:
                 out.add(i)
         return out
 
-    def inputs_to_sign(self):
-        out = set()
-        for txin in self.inputs():
-            num_sig = txin.get('num_sig')
-            if num_sig is None:
-                continue
-            x_signatures = txin['signatures']
-            signatures = filter(None, x_signatures)
-            if len(signatures) == num_sig:
-                # input is complete
-                continue
-            for k, x_pubkey in enumerate(txin['x_pubkeys']):
-                if x_signatures[k] is not None:
-                    # this pubkey already signed
-                    continue
-                out.add(x_pubkey)
-        return out
 
     def sign(self, keypairs):
         for i, txin in enumerate(self.inputs()):
@@ -833,7 +790,8 @@ class Transaction:
         self.deserialize()
         out = {
             'hex': self.raw,
-            'complete': self.is_complete()
+            'complete': self.is_complete(),
+            'final': self.is_final(),
         }
         return out
 
@@ -853,8 +811,8 @@ class Transaction:
         threshold = 57600000
         weight = 0
         for txin in self.inputs():
-            age = wallet.get_confirmations(txin["prevout_hash"])[0]
-            weight += txin["value"] * age
+            height, conf, timestamp = wallet.get_tx_height(txin["prevout_hash"])
+            weight += txin["value"] * conf
         priority = weight / size
         print_error(priority, threshold)
 
